# calculate the peak range for threshold methods from simData
peak_range = function(table, threshold) {
  ws = c(0.25,0.5,0.75,1.0)
  result = data.frame()
  for(i in 1:4) {
    tmp = subset(table, subset = (Weight == ws[i] & Measure=="ARI"))
    for(x in unique(table$Definition)) {
      tmp2 = subset(tmp, subset = (Definition == x))
      ml = max(tmp2$Value)
      id = tmp2$Cutpoint[which(tmp2$Value == ml)]
      idx = tmp2$Cutpoint[which(tmp2$Value <= threshold*ml)]
      result = rbind(result, data.frame(weight = ws[i],minrge = max(max(idx[which(idx<id)]),min(tmp2$Cutpoint)), maxrge = min(min(idx[which(idx>id)]),max(tmp2$Cutpoint)),bestcut = id, definition = x))
    }
  }
  result
}

# summarize overlap/differences in peak ranges across A,B,C networks
peak_summary = function(A_rge,B_rge,C_rge) {
  alldat = list(A=A_rge,B=B_rge,C=C_rge)
  n=list(AinB=0, AinC=0, BinA=0, BinC=0, CinA=0, CinB=0)
  avl = list(A=0, B=0, C=0)
  overl = list(AB=0, AC=0, BC=0)
  for(j in 1:length(A_rge$bestcut)) {
    for(i in 1:3) {
      avl[[i]] =avl[[i]] + alldat[[i]]$maxrge[j] - alldat[[i]]$minrge[j]
    }
    for(i in 1:2) {
      for(i2 in (i+1):3) {
        x = min(alldat[[i]]$maxrge[j],alldat[[i2]]$maxrge[j]) - max(alldat[[i]]$minrge[j],alldat[[i2]]$minrge[j])
        overl[[i+i2-2]] = overl[[i+i2-2]] + max(x,0)
      }
    }
    k=1
    for(i in 1:3) {
      for(i2 in 1:3) {
        if(i==i2) next
        n[[k]] = n[[k]] + if(alldat[[i]]$bestcut[j]>=alldat[[i2]]$minrge[j] && alldat[[i]]$bestcut[j]<=alldat[[i2]]$maxrge[j]) 1 else 0
        k=k+1
      }
    }
  }
  for(j in 1:length(avl)) avl[[j]] = avl[[j]]/length(A_rge$bestcut)
  for(j in 1:length(overl)) overl[[j]] = overl[[j]]/length(A_rge$bestcut)
  list(n=n,avl=avl,overl=overl)
}