# full cluster analysis on networks A,B,C
# saves true clusters and inferred clusters as well as resulting ARIs
cluster_analysis = function(dir,datadir=NULL, destdir="../HIVclusters/", outname = "", limit =0) {
  library(data.table)
  if(is.null(datadir)) datadir = dir
  subd = list("A", "B", "C")
  for(sd in subd) {
    table = list()
    clusters = list()
    scores = list()
    path = paste0(dir,"/",sd,"/")
    if(!dir.exists(path)) next
    load(paste0(datadir,"/",sd,"_simData.RData"))
    simD = get(paste0(sd,"_simData"))
    
    for(i in 1:4) {
      name = paste0(sd,"_IS_",i)
      benchd <- NULL
      clusters[[i]] = list()
      for(id in 1:300) {
        jobres = paste0(path,"/","job_",name,"_",id,"_.RData")
        x = try(load(jobres))
        if(class(x)=="try-error") {
          clusters[[i]][[id]] = "no file found"
          next
        }
        benchd <- rbind(benchd, b)
        clusters[[i]][[id]] = .get_clusters_ind(b,simD[[i]][[id]])
      }
      if(length(benchd$id)>1) {
        benchd <- benchd[order(benchd$id)]
        setkey(benchd, id)
      }
      table[[i]] = benchd
      scores[[i]] = .get_scores(clusters[[i]], limit = limit)
    }
    
    resnames = c(paste0(sd,outname,"_full_res"),paste0(sd,outname,"_clusters"),paste0(sd,outname,"_scores"))
    assign(resnames[1],table)
    assign(resnames[2],clusters)
    assign(resnames[3],scores)
    save(list=resnames,file=paste0(destdir,"/",sd,outname,"_ML_results.RData"))
    save(list=resnames[-1],file=paste0(destdir,"/",sd,outname,"_ML_scores.RData"))
  }
}

# get true cluster repartition from simData and inferred cluster repartition from ML output
.get_clusters_ind = function(vect, simd) {
  tree = vect$tree[[1]]
  nisl = simd$contactNet[[9]][[2]]$islands.size
  shifts = try(vect[[length(vect)]][[1]]$shifts.edge[-1])
  if(class(shifts) == "try-error") { 
    print(i)
    return("error in shifts")
  }
  states = rep(1,length(tree$tip.label))
  k = 2
  for(e in shifts) {
    desc = e
    while(length(desc)>0) {
      x = tree$edge[desc[1],2]
      if(x<=length(tree$tip.label)) states[x]=k
      else desc=c(desc,which(tree$edge[,1]==x))
      desc=desc[-1]
    }
    k = k+1
  }
  real = .realclusters(simd,tree$tip.label)
  result = list(labels = tree$tip.label,clusters=states,realclusters=real)
}

# get ARI between true and inferred clusters
.get_scores = function(clusters, limit =0) {
  library(fpc)
  m = length(clusters)
  temp = c()
  n = 0
  for(j in 1:m) {
    if(class(clusters[[j]])=="character") {
      temp[j] = NA
    }
    else {
      smcl = as.integer(names(which(table(clusters[[j]]$realclusters)<limit)))
      idx = -which(clusters[[j]]$realclusters %in% smcl)
      #print(length(clusters[[j]]$labels[idx]))
      if(length(idx)==0) idx = 1:length(clusters[[j]]$realclusters)
      if(length(unique(clusters[[j]]$realclusters[idx]))==1 || length(unique(clusters[[j]]$clusters[idx]))==1) {
        temp[j] = NA
        n = n+1
        next
      }
      temp[j] = cluster.stats(d = dist(clusters[[j]]$labels[idx]), 
                              clustering = clusters[[j]]$clusters[idx], 
                              alt.clustering = clusters[[j]]$realclusters[idx],
                              compareonly = T)$corrected.rand
    }
  }
  x = mean(temp,na.rm = T)
  temp[which(is.na(temp))]=0
  x_wo = mean(temp)
  res = list(scores = temp, mean = x, failed = n, mean_wofailed = x_wo) #,ESS = clusters$ESS$npb)
  res
}

# get true clusters from simData
.realclusters = function(data,labels) {
  nisl = data$contactNet[[9]][[2]]$islands.size
  if(!is.null(nisl)) return((as.numeric(labels)-1)%/%nisl + 1)
  real = data$clusterIndices[as.numeric(labels)]
  real
}