# adapted drop.tip function (from ape) to record states
drop.tip_wrecords <- function(phy, tip, trim.internal = TRUE, root.edge = 0, rooted = is.rooted(phy)) {
  if (!inherits(phy, "phylo"))
    stop('object "phy" is not of class "phylo"')
  Ntip <- length(phy$tip.label)
  ## find the tips to drop:
  if (is.character(tip))
    tip <- which(phy$tip.label %in% tip)
  out.of.range <- tip > Ntip
  if (any(out.of.range)) {
    warning("some tip numbers were larger than the number of tips: they were ignored")
    tip <- tip[!out.of.range]
  }
  if (!length(tip)) return(phy)
  
  wbl <- !is.null(phy$edge.length)
  
  phy <- reorder(phy)
  NEWROOT <- ROOT <- Ntip + 1
  Nnode <- phy$Nnode
  Nedge <- dim(phy$edge)[1]
  
  edge1 <- phy$edge[, 1] # local copies
  edge2 <- phy$edge[, 2] #
  keep <- !logical(Nedge)
  nodes_rm = c()
  
  ## delete the terminal edges given by `tip':
  keep[match(tip, edge2)] <- FALSE
  
  if (trim.internal) {
    ints <- edge2 > Ntip
    ## delete the internal edges that do not have anymore
    ## descendants (ie, they are in the 2nd col of `edge' but
    ## not in the 1st one)
    repeat {
      sel <- !(edge2 %in% edge1[keep]) & ints & keep
      if (!sum(sel)) break
      keep[sel] <- FALSE
      nodes_rm = c(nodes_rm, edge2[sel])
    }
    if (root.edge && wbl) {
      degree <- tabulate(edge1[keep])
      if (degree[ROOT] == 1) {
        j <- integer(0) # will store the indices of the edges below the new root
        repeat {
          i <- which(edge1 == NEWROOT & keep)
          j <- c(i, j)
          NEWROOT <- edge2[i]
          degree <- tabulate(edge1[keep])
          if (degree[NEWROOT] > 1) break
        }
        keep[j] <- FALSE
        if (length(j) > root.edge) j <- 1:root.edge
        NewRootEdge <- sum(phy$edge.length[j])
        if (length(j) < root.edge && !is.null(phy$root.edge))
          NewRootEdge <- NewRootEdge + phy$root.edge
        phy$root.edge <- NewRootEdge
      }
    }
  }
  
  if (!root.edge) phy$root.edge <- NULL
  
  ## drop the edges
  phy$edge <- phy$edge[keep, ]
  if (wbl) phy$edge.length <- phy$edge.length[keep]
  if(length(nodes_rm)) phy$node.label = phy$node.label[-match(nodes_rm, phy$node.label)]
  
  ## find the new terminal edges (works whatever 'subtree' and 'trim.internal'):
  TERMS <- !(phy$edge[, 2] %in% phy$edge[, 1])
  
  ## get the old No. of the nodes and tips that become tips:
  oldNo.ofNewTips <- phy$edge[TERMS, 2]
  
  n <- length(oldNo.ofNewTips) # the new number of tips in the tree
  
  ## the tips may not be sorted in increasing order in the
  ## 2nd col of edge, so no need to reorder $tip.label
  phy$edge[TERMS, 2] <- rank(phy$edge[TERMS, 2])
  phy$tip.label <- phy$tip.label[-tip]
  
  phy$Nnode <- dim(phy$edge)[1] - n + 1L # update phy$Nnode
  
  aux = function(l) {
    if(l <= length(phy$tip.label)) l
    else {
      which(phy$node.label==l) + length(phy$tip.label)
    }
  }
  
  phy$edge[,1] = sapply(phy$edge[,1], aux)
  phy$edge[,2] = sapply(phy$edge[,2], aux)
  
  phy = .collapse.singles(phy)
  phy
}