# adapted reconstruct function (from TreeSim) to record states
reconstruct_wrecords <- function(phy,frac=1,stochastic = F,drop_extinct = T, save_labels = F, drop_extant = F){
  recordt = phy$tip.states
  recordn = phy$node.states
  
  if(drop_extinct && drop_extant) stop("Current settings will remove the tree entirely")
  
  if (class(phy)=="phylo" && drop_extinct){
    extinct=c()
    d = max(node.depth.edgelength(phy))-node.depth.edgelength(phy)
    ntotal<- length(phy$tip.label)
    for(i in 1:ntotal) {
      if(d[i]>0.00001) extinct = c(extinct,i)
    }
    if(length(extinct)==ntotal-1) {phy = 1}
    else if(length(extinct)==ntotal) {phy = 0}
    else {
      phy = drop.tip_wrecords(phy,extinct)
    }
  }
  if (!class(phy)=="phylo") return(NULL)
  
  if (class(phy)=="phylo" && drop_extant){
    extant=c()
    d = max(node.depth.edgelength(phy))-node.depth.edgelength(phy)
    ntotal<- length(phy$tip.label)
    for(i in 1:ntotal) {
      if(d[i]<0.00001) extant = c(extant,i)
    }
    if(length(extant)==ntotal-1) {phy = 1}
    else if(length(extant)==ntotal) {phy = 0}
    else {
      phy = drop.tip_wrecords(phy,extant)
    }
  }
  if (!class(phy)=="phylo") return(NULL)
  
  if(!stochastic) {
    ntotal<- length(phy$tip.label)
    delete <- round((1-frac)*ntotal)
    if (ntotal <= delete + 1) return(NULL)
    if (delete>0){
      droptips = sample(ntotal,delete)
      phy<-drop.tip_wrecords(phy,droptips)
    }
  }
  if(stochastic) {
    if(drop_extinct && length(extinct)>0) frac = frac[-extinct]
    ntotal<- length(phy$tip.label)
    if(length(frac)!=ntotal) print("Error in reconstruct, sampling rates do not match tips")
    else {
      droptips = c()
      for(i in 1:ntotal) {
        x = runif(1,0,1)
        if(x>frac[i]) droptips = c(droptips,i)
      }
      if(length(droptips >0)) {
        phy = drop.tip_wrecords(phy,droptips)
        frac = frac[-droptips]
      }
      phy$nf = frac
    }
  }
  
  if (!class(phy)=="phylo") return(NULL)
  phy$tip.states = c()
  for(j in 1:length(phy$tip.label)) {
    id = phy$tip.label[j]
    if(id>length(recordt)) phy$tip.states[j] = recordn[id-length(recordt)]
    else phy$tip.states[j] = recordt[id]
    if(is.na(phy$tip.states[j])) browser()
  }
  
  phy$node.states = c()
  for(j in 1:phy$Nnode) {
    id = phy$node.label[j]
    phy$node.states[j] = recordn[id-length(recordt)]
    if(is.na(phy$node.states[j])) browser()
  }
  ntips = length(phy$tip.label)
  if(!save_labels) {
    phy$tip.label = 1:ntips
    phy$node.label = (ntips+1):(ntips+phy$Nnode)
  }
  phy
}
