# simulate trees under forward birth-death model with state changes and exponential decay
sim_exp_bd = function(lambda, lambdarate, mu, gamma, ntrees, stepsize, ntips = NULL, age = NULL,
                      mufix = T, stt = T, remove_cst = F, remove_extant = T, min_ntips = NULL, sigma = 1) {
  library(ape)
  phy = list()
  while(length(phy)<ntrees) {
    nsim = ntrees-length(phy)
    print(nsim)
    phy = c(phy,lapply(1:nsim,function(i) {one_exp_bd_sim(lambda, lambdarate, mu, gamma, ntips, age, stepsize, 
                                                        mufix=mufix, stt=stt, remove_extant = remove_extant, sigma = sigma)}))
    for(i in 1:ntrees) {
      if(class(phy[[ntrees+1-i]]$phylo) != "phylo" || (remove_cst && length(phy[[ntrees+1-i]]$record$shifts.edge)<2)
         || (!is.null(min_ntips) && length(phy[[ntrees+1-i]]$phylo$tip.label) < min_ntips)) {
        phy = phy[-(ntrees+1-i)]
      }
    }
  }
  phy
}

one_exp_bd_sim = function(lambda, lambdarate, mu, gamma, ntips, age, stepsize, mufix = T, stt = T, remove_extant = T, sigma = 1) {
  edge <- c(-1,-2)  	#matrix of edges
  edge<-rbind(edge,c(-1,-3))
  leaves <- c(-2,-3)			#list of extant leaves
  timecreation <-c(0,0,0)		#time when species -2, -3, ... -n was created after origin
  maxspecies <- -3		#smallest species
  edge.length <- c(0,0)		#edge length. if 0: leaf which didn't speciate /extinct yet
  lambdatab = lambda()
  mutab = mu()
  
  if(mufix) {
    mu = function() {mutab[1]}
  }
  
  t0 = 0
  lrtab = lambdarate()
  states = c(1,1)
  
  extinct <- vector()		#list of extinct leaves
  time <-0				#time after origin
  record = list(shifts.edge=0,shifts.time=0,gamma=gamma)
  
  while (TRUE){
    all_l = lambdatab[states]*exp(lrtab[states]*(t0[states]-time))
    all_m = mutab[states]
    totalrate = length(leaves)*gamma+sum(all_l)+sum(all_m)
    timestep <- rexp(1,totalrate)    #time since last event
    if(timestep > stepsize) {
      time = time + stepsize
    }
    else {
      time = time+timestep			#time after origin
      specevent <- runif(1,0,1)  	#event speciation or extinction
      speciation = TRUE
      
      #choice of species and event here
      stoploop = FALSE
      i=1
      cutoff=0
      while(!stoploop && i<=length(leaves)) {
        cutoff = cutoff + all_l[i]/totalrate
        if(specevent<cutoff) {
          stoploop = TRUE
          species = leaves[i]
        }
        else{i=i+1}
      }
      if(!stoploop) {i=1}
      while(!stoploop && i<=length(leaves)) {
        cutoff = cutoff + all_m[i]/totalrate
        if(specevent<cutoff) {
          stoploop = TRUE
          speciation = FALSE
          species = leaves[i]
        }
        else{i=i+1}
      }
      
      if(!stoploop) { #then we are in rate shift event
        species <- sample(leaves,1)
        del <- which(leaves == species)
        edgeevent <- which(edge == species) - length(edge.length)
        
        lambdatab = c(lambdatab, lambda())
        mutab = c(mutab, mu())
        lrtab = c(lrtab, lambdarate())
        t0 = c(t0, time)
        states[del] = length(lambdatab)
        record$shifts.edge = c(record$shifts.edge,edgeevent)
        record$shifts.time = c(record$shifts.time,time)
      }
      else {
        if(speciation) {
          del <- which(leaves == species)        
          edgespecevent <- which(edge == species) - length(edge.length)
          edge.length[edgespecevent] <- time-timecreation[- species]
          edge <- rbind(edge,c(species,maxspecies-1))
          edge <- rbind(edge,c(species,maxspecies-2))
          edge.length <- c(edge.length,0,0)
          leaves <- c(leaves,maxspecies-1,maxspecies-2)
          states = c(states,states[del],states[del])
          maxspecies <- maxspecies-2
          leaves <- leaves[- del]
          states = states[- del]
          timecreation <- c(timecreation,time,time)
          
          if(stt) totaltips = length(leaves) + length(extinct)
          else totaltips = length(leaves)
          if (!remove_extant && !is.null(ntips) && round(totaltips*(1-sigma)) == totaltips-ntips){break}
        } else {
          if (length(leaves) == 1) {
            return(NULL) #extinct tree
          }
          del <- which(leaves == species)
          edgespecevent <- which(edge == species) - length(edge.length)
          extinct <- c(extinct,leaves[del])
          leaves <- leaves[- del]
          states = states[- del]
          edge.length[edgespecevent] <- time-timecreation[- species]
          if(remove_extant && !is.null(ntips) && round(length(extinct)*(1-sigma)) == length(extinct)-ntips){break}
          if (length(leaves) == 0) break
        }
      }
    }
    if(!is.null(age) && time > age) break
  }
  
  if (length(leaves) != 0) {
    #length pendant edge
    while(T) {
      all_l = lambdatab[states]*exp(lrtab[states]*(t0[states]-time))
      all_m = mutab[states]
      totalrate = length(leaves)*gamma+sum(all_l)+sum(all_m)
      step = rexp(1,totalrate)			#time after origin
      if(step > stepsize) time = time + stepsize
      else {
        time = time + step
        break
      }
    }
    
    #assign pendant edge length
    for (j in (1:length(leaves))){
      k = which( edge == leaves[j]  ) - length(edge.length)
      edge.length[ k ] <- time - timecreation[- leaves[j]]
    }
  }
  
  ntips = length(leaves) + length(extinct)
  
  nodes <- ntips*2 -1
  leaf=1
  interior=ntips+1
  for (j in (1:nodes)){
    if (sum(match(leaves,- j,0)) + sum(match(extinct,- j,0)) == 0) { #node
      posvalue <- interior
      interior <- interior +1
    } else { #leaf
      posvalue <- leaf
      leaf <- leaf +1
    }
    replacel <- which(edge == - j)
    edge[replacel] = posvalue
  }       
  
  states = rep(1,nodes)      
  if(length(record$shifts.edge)>1) {
    for(i in 2:length(record$shifts.edge)) {
      desc = record$shifts.edge[i] #edge concerned                   
      while(length(desc)>0) {
        states[edge[desc[1],2]]=i
        desc=c(desc,which(edge[,1]==edge[desc[1],2]))
        desc=desc[-1]
      }
    }
  }
  phy <- list(edge = edge, edge.length = edge.length, Nnode = ntips-1)
  phy$tip.states = states[1:ntips]
  phy$node.states = states[(1+ntips):nodes]
  phy$tip.label = 1:ntips
  phy$node.label = (1+ntips):nodes
  class(phy) <- "phylo"
  
  record$lambdas = lambdatab
  record$mus = mutab
  record$lambda_rates = lrtab
  
  res = list(phylo=phy,record=record)
  res$phylo = .wrap.collapse.singles(res$phylo)
  
  res$phylo <- reconstruct_wrecords(res$phylo, frac = sigma, drop_extinct = !stt, drop_extant = remove_extant)
  if (!class(res$phylo)=="phylo") return(NULL)
  
  #record management here
  res$phylo = .wrap.reorder.phylo(res$phylo)  
  res = update_records(res)
  
  res
}