# summarize results of MSBD parameter inference on a simulated dataset
analyze_simbd_list = function(dir,treefile,suffix="", minsize = 0) {
  library(ape)
  library(tools)
  
  load(treefile)
  treefile = file_path_sans_ext(basename(treefile))
  tree_list = get(treefile)
  n = length(tree_list)
  
  list_result = list()
  k = 1
  for(i in n:1) {
    nm = paste0(dir,"/job_",treefile,"_",i,suffix,".RData")
    if(!file.exists(nm)) {
      tree_list = tree_list[-i]
    }
    else {
      load(nm)
      list_result[[k]] = get("result")
      #the tree read as input may not be the tree as in the list. because ape
      tmp = read.tree(paste0(dir,"/",treefile,"_",i,".tre"))
      tmp$edge = matrix(as.integer(c(tmp$tip.label,tmp$node.label)[tmp$edge]),ncol=2)
      
      for(x in 1:length(list_result[[k]]$shifts.edge)) {
        if(list_result[[k]]$shifts.edge[x] == 0) next
        edge_ns = tmp$edge[list_result[[k]]$shifts.edge[x],]
        cor_edge = which(tree_list[[i]]$phylo$edge[,2]==edge_ns[2])
        list_result[[k]]$shifts.edge[x] = cor_edge
      }
      
      k = k +1
    }
  }
  list_result = rev(list_result)
  
  n = length(list_result)
  if(n == 0) return(NULL)
  #average discrepancy in # of shifts detected
  disc = 0
  posd = 0
  negd = 0
  posd_mins = 0
  negd_mins = 0
  disc_mins = 0
  #shifts correctly detected
  detect = 0
  recorded = 0
  recorded_mins = 0
  inters = 0
  inters_mins = 0
  #correct configurations found
  exact = 0
  exact_mins = 0
  ex.l = c()
  ex.s = c()
  ex = c()
  ex_wo = c()
  ex.l_wo = c()
  ex.s_wo = c()
  #records of correct shifts
  correct.i = c()
  correct.l = c()
  correct.i_wo = c()
  correct.l_wo = c()
  #average likelihood of correct configs
  meanl_c = 0
  meanl_c_wo = 0
  #size of shifts
  discreps = list(est.lbda=0,est.mu=0,rec.lbda=0,rec.mu=0,
                  cor.lbda=0,cor.mu=0,rec.cor.lbda=0,rec.cor.mu=0)
  edge_val = list(lbda = list(est = list(), rec = list()))#, mu = list(est = list(), rec = list()))
  sampled_extants = 0
  
  for(i in 1:n) {
    frecords = tree_list[[i]]$record
    festims = list_result[[i]]
    tmp = .calculate_discreps(tree_list[[i]]$phylo,frecords,festims)
    
    discreps$est.lbda=discreps$est.lbda+tmp$est.lbda
    discreps$est.mu=discreps$est.mu+tmp$est.mu
    discreps$rec.lbda=discreps$rec.lbda+tmp$rec.lbda
    discreps$rec.mu=discreps$rec.mu+tmp$rec.mu
    discreps$cor.lbda=discreps$cor.lbda+tmp$cor.lbda
    discreps$cor.mu=discreps$cor.mu+tmp$cor.mu
    discreps$rec.cor.lbda=discreps$rec.cor.lbda+tmp$rec.cor.lbda
    discreps$rec.cor.mu=discreps$rec.cor.mu+tmp$rec.cor.mu
    
    estims = festims$shifts.edge[-1]
    records = frecords$shifts.edge[-which(tree_list[[i]]$record$shifts.edge==0)]
    
    ntips = length(tree_list[[i]]$phylo$tip.label)
    times = node.depth.edgelength(tree_list[[i]]$phylo)
    sampled_extants = sampled_extants + length(which(max(times)-times < 1e-5))/n
    
    records_mins = c()
    if(length(records) > 0) {
      offs = length(which(tree_list[[i]]$record$shifts.edge==0))
      for(ii in 1:length(records)) {
        if(length(which(tree_list[[i]]$phylo$tip.states == ii + offs)) > minsize) {
          records_mins = c(records_mins, records[ii])
        }
      }
    }
    
    a = length(estims)
    b = length(records)
    b2 = length(records_mins)
    if(a<b) {
      negd = negd+1
      disc = disc+(b-a) 
    }
    if(a>b) {
      posd = posd+1
      disc = disc+(a-b)
    }
    if(a<b2) {
      negd_mins = negd_mins+1
      disc_mins = disc_mins+(b2-a) 
    }
    if(a>b2) {
      posd_mins = posd_mins+1
      disc_mins = disc_mins+(a-b2)
    }
    
    detect = detect + a
    recorded = recorded + b
    recorded_mins = recorded_mins + b2
    
    u = which(estims %in% records)
    inters = inters + length(u)
    correct.i = c(correct.i, rep(i,length(u)))
    correct.l = c(correct.l, rep(list_result[[i]]$likelihood,length(u)))
    if(a==b && length(u)==a) {
      exact = exact+1
      meanl_c = meanl_c + list_result[[i]]$likelihood
      ex = c(ex,i)
      ex.l = c(ex.l, list_result[[i]]$likelihood)
      ex.s = c(ex.s,b)
    }
    
    u2 = which(estims %in% records_mins)
    inters_mins = inters_mins + length(u2)
    correct.i_wo = c(correct.i_wo, rep(i,length(u2)))
    correct.l_wo = c(correct.l_wo, rep(list_result[[i]]$likelihood,length(u2)))
    if(a==b2 && length(u2)==a) {
      exact_mins = exact_mins+1
      ex_wo = c(ex_wo,i)
      meanl_c_wo = meanl_c_wo + list_result[[i]]$likelihood
      ex.l_wo = c(ex.l_wo, list_result[[i]]$likelihood)
      ex.s_wo = c(ex.s_wo,b2)
    }
  }
  meanl_c = meanl_c/exact
  meanl_c_wo = meanl_c_wo/exact_mins
  
  #weighted mu, lambda difference
  ldiff = 0
  l_est = list(estim = 0,record = 0)
  mdiff = 0
  m_est = list(estim = 0,record = 0)
  g_estim=0
  for(i in 1:n) {
    ml = list_result[[i]]
    a = .weightedlm_f(tree_list[[i]], rev.time = T)
    b = .weightedlm_f(list(phylo = tree_list[[i]]$phylo, record=ml))
    ldiff[i] = abs(a$lambda-b$lambda)
    mdiff = mdiff + abs(a$mu-b$mu)
    l_est$estim[i] = b$lambda
    l_est$record[i] = a$lambda
    m_est$estim[i] = b$mu
    m_est$record[i] = a$mu
    g_estim = g_estim + list_result[[i]]$gamma
    edge_val$lbda$rec[[i]] = a$lbda_edges
    edge_val$lbda$est[[i]] = b$lbda_edges
  }  
  ldiff = ldiff
  mdiff = mdiff/n
  
  #confidence interval
  upconf = c()
  lowconf = c()
  av_width = 0 #corrected for lik
  av_up = 0
  av_low = 0
  for(i in 1:n) {
    conf = list_result[[i]]$best_models
    l = list_result[[i]]$likelihood
    if(length(conf) > 2) { lowl = conf[length(conf)-2] }
    else { lowl = 0 }
    upl = conf[length(conf)]
    upconf = c(upconf,upl-l)
    lowconf = c(lowconf, lowl-l)
    av_up = av_up + (upl-l)/l
    av_low = av_low + (lowl-l)/l
    av_width = av_width + (upl+lowl-2*l)/l
  }
  av_width = av_width/n
  av_up = av_up/n
  av_low = av_low/n
  
  discreps$est.lbda=discreps$est.lbda/detect
  discreps$est.mu=discreps$est.mu/detect
  discreps$rec.lbda=discreps$rec.lbda/recorded
  discreps$rec.mu=discreps$rec.mu/recorded
  discreps$cor.lbda=discreps$cor.lbda/inters
  discreps$cor.mu=discreps$cor.mu/inters
  discreps$rec.cor.lbda=discreps$rec.cor.lbda/inters
  discreps$rec.cor.mu=discreps$rec.cor.mu/inters
  
  result = list(shifts = list(averdisc = disc/n, overest = posd, underest = negd, detected = detect, recorded = recorded), 
                shifts.mins = list(mins = minsize, averdisc = disc_mins/n, overest = posd_mins, underest = negd_mins, recorded = recorded_mins),
                correct = list(list = correct.i, likelihoods = correct.l, total = inters), 
                correct.mins = list(list = correct.i_wo, likelihoods = correct.l_wo, total = inters_mins),
                exact = list(list = ex, likelihoods = ex.l, shifts = ex.s, total = exact, avlik = meanl_c), 
                exact.mins = list(list = ex_wo, likelihoods = ex.l_wo, shifts = ex.s_wo, total = exact_mins, avlik = meanl_c_wo),
                confidence = list(lower.list = lowconf, upper.list = upconf, lower.av = av_low, upper.av = av_up, width.av = av_width),
                lambda.weighted = l_est, mu.weighted = m_est, lambda.aver.diff = list(val = ldiff, med = median(ldiff)), mu.aver.diff = mdiff, gamma.aver=g_estim/n,
                discrepancies = discreps, sampled_extants = sampled_extants, edge_val = edge_val)
  result
}