# plot comparison between MSBD and cutpoint-based methods from saved results files
build_all_graphs_HIV = function(outdir, scoresdir, datadir,scale=T) {
  let = c("A","B","C")
  for(l in let) {
    load(paste0(datadir,"/",l,"_data_all.RData"))
    load(paste0(datadir,"/",l,"_data_def3.RData"))
    alld = get("data_all")
    def3 = get("data_def3")
    
    x=try(load(paste0(scoresdir,"/",l,"_precalc_ML_scores.RData")))
    if(class(x)=="try-error") {
      load(paste0(scoresdir,"/",l,"_fast_ML_scores.RData"))
      load(paste0(scoresdir,"/",l,"_fast_lim8_ML_scores.RData"))
      sc = get(paste0(l,"_fast_scores"))
      scl = get(paste0(l,"_fast_lim8_scores"))
    }
    else {
      load(paste0(scoresdir,"/",l,"_prec_lim8_ML_scores.RData"))
      sc = get(paste0(l,"_precalc_scores"))
      scl = get(paste0(l,"_prec_lim8_scores"))
    }
    
    .build_graph_HIV(outdir,alld,def3,sc, scl, l,scale)
  }
}

.build_graph_HIV = function(outdir,data_all, data_def3, scores, scores_lim8, letter, scale3 = T) {
  ## The graphs are produced with ggplot2. Make sure it is installed.
  library(ggplot2)
  
  weights = c(0.25,0.5,0.75,1.0)
  
  x = seq(min(data_all$Cutpoint),max(data_all$Cutpoint),0.005)
  for(i in 1:4) {
    data_all <- rbind(data_all, data.frame(Cutpoint = x, Value = scores[[i]]$mean, Weight = weights[i], Definition = "MSBD", Measure = "ARI"))
    data_all <- rbind(data_all, data.frame(Cutpoint = x, Value = scores_lim8[[i]]$mean, Weight = weights[i], Definition = "MSBD limited", Measure = "ARI"))
  }
  
  ## The scalingFactor applies a scaling factor to the x values from the original def3.
  if(scale3) {
    scalingFactora <- (max(data_all$Cutpoint)-min(data_all$Cutpoint))/(max(data_def3$Cutpoint)-min(data_def3$Cutpoint))
    scalingFactorb = min(data_all$Cutpoint) - min(data_def3$Cutpoint)*scalingFactora
    data_def3$Cutpoint <- data_def3$Cutpoint*scalingFactora + scalingFactorb
  }
  data_def3$Definition = "Def. 3"
  data_all <- rbind(data_all, data_def3)
  
  ## To fix the order of the items in the graph legend!
  data_all$Definition <- factor(data_all$Definition, levels = c("Def. 1", "Def. 2", "Def. 3", "Def. 4: Average link.", "Def. 4: Complete link.","Def. 4: WPGMA", "MSBD","MSBD limited"))
  
  ## The following functions plot the graphs!
  ## The calls to ggsave will output each graph to a different file.
  ## Play with legend.position if you want to move the legend in the graph
  ## coord_cartesian will change the range of the y axis.
  for(i in 1:4) {
    p2 <- ggplot(subset(data_all, subset = (Measure == "ARI" & Weight == weights[i])), aes(x = Cutpoint, y = Value, group = Definition))
    ylim = ceiling(max(subset(data_all, subset = (Measure == "ARI" & Weight == weights[i]))$Value)*10)/10
    p2 <- p2 + geom_line(aes(colour = Definition,linetype=Definition)) + geom_point(aes(colour = Definition)) + theme_bw() + coord_cartesian(ylim = c(0,ylim)) + theme(legend.position = "bottom") + theme(text = element_text(size=15), legend.box.margin = margin(l = -50))
    p2 = p2 + scale_linetype_manual(values=c(rep(1,7), 3))+ scale_color_manual(values=c("#F8766D", "#C49A00", "#53B400", "#00C094", "#00B6EB", "#A58AFF", "#FB61D7", "#FB61D7"))
    p2 = p2 + ylab("Adjusted Rand Index")
    if(scale3) p2 = p2 + scale_x_continuous(sec.axis = sec_axis(~(.-scalingFactorb)/scalingFactora, name = "Cutpoint (Def 3.)"))
    ggsave(paste0(outdir,"/netw",letter,"_w=",weights[i],".pdf")) 
  }
}