# plot tree, ML clusters and ML parameters obtained from MSBD inference
plot_MSBD_results = function(result, plotfile, tree = NULL, treefile = NULL, no_labels = F) {
  library(ape)
  if(is.null(tree)) tree = read.tree(treefile)
  pdf(plotfile)
  colors = c("darkgreen","black","blue","red","green","gold","purple","orange","cyan","deeppink")
  states = .tree_states(tree,result$shifts.edge[-1])
  edge.colors = colors[states[tree$edge[,1]]%%10+1]
  par(xpd = NA, mar = par()$mar + c(2, 0, 0, 0))
  plot.phylo(tree, edge.color = edge.colors,cex = 0.25, show.tip.label = !no_labels)
  txt = c()
  cols = c()
  for(i in 1:length(result$shifts.edge)) {
    cols = c(cols, colors[i%%10+1])
    if(!is.null(result$lambda_rates)) {
      sbs = substitute(expression(paste(lambda[0*","*i],"=",a,", ", z[i],"=", b,", ",mu[i],"=", c)),
                       list(i=i,a=formatC(result$lambdas[i],digits=2,format="g"),b=formatC(result$lambda_rates[i],digits=2,format="g"),c=formatC(result$mus[i],digits=2,format="g")))
    }
    else {
      sbs = substitute(expression(paste(lambda[i],"=",a,", ",mu[i],"=", c)),
                          list(i=i,a=formatC(result$lambdas[i],digits=2,format="g"),c=formatC(result$mus[i],digits=2,format="g")))
    }
    txt = c(txt, eval(sbs))
  }
  legend(2,-20,legend = txt,col = cols, cex = 0.75,lty = 1)
  dev.off()
}

.tree_states = function(tree,edges) {
  states = rep(1,tree$Nnode + length(tree$tip.label))
  k = 2
  for(e in edges) {
    desc = e
    while(length(desc)>0) {
      x = tree$edge[desc[1],2]
      states[x]=k
      desc=c(desc,which(tree$edge[,1]==x))
      desc=desc[-1]
    }
    k = k+1
  }
  states
}