# plot tree and clusters obtained from Cluster Picker
plot_clusterpicker = function(treefile, plotfile, no_labels = F) {
  library(ape)
  tree = read.tree(treefile)
  ntips = length(tree$tip.label)
  states = rep(-1,ntips+tree$Nnode)
  for(i in 1:ntips) {
    lab = tree$tip.label[i]
    x = strsplit(lab, "_")[[1]]
    if(substr(x[1],1,5)=="Clust") {
      tree$tip.label[i] = substr(lab,nchar(x[1])+2,nchar(lab))
      states[i] = as.integer(substr(x[1],6,nchar(x[1])))
    }
    else states[i] = 0
  }
  states = .prop_states(tree, states)
  
  pdf(plotfile)
  colors = c("darkgreen","blue","red","green","gold","purple","orange","cyan","deeppink","slategray")
  edgestates = states[tree$edge[,1]]
  edge.colors = colors[edgestates%%10+1]
  edge.colors[which(edgestates==0)] = "black"

  plot.phylo(tree, edge.color = edge.colors,cex = 0.25, show.tip.label = !no_labels)
  dev.off()
}

.prop_states = function(tree, states) {
  ntips = length(tree$tip.label)
  while(length(which(states==-1))>0) {
    for(i in 1:tree$Nnode) {
      if(states[i+ntips] != -1) next
      desc = tree$edge[which(tree$edge[,1]==ntips+i),2]
      if(states[desc[1]] == -1 || states[desc[2]] == -1) next
      if(states[desc[1]] ==  states[desc[2]]) states[i+ntips] = states[desc[1]]
      else states[i+ntips] = 0
    }
  }
  states
}