# plot tree and clusters obtained from Phylo Part
plot_phylopart = function(treefile, clusterfile, plotfile = NULL, no_labels = T) {
  library(ape)
  tree = read.tree(treefile)
  ntips = length(tree$tip.label)
  states = rep(-1,ntips+tree$Nnode)
  clusters = read.csv(clusterfile,stringsAsFactors = F)
  clusternames = unique(clusters$clustername)
  clusternames = clusternames[-which(clusternames == 0)]
  for(i in 1:ntips) {
    lab = tree$tip.label[i]
    if(lab %in% clusters$leafname) {
      idx = which(clusters$leafname == lab)
      if(clusters$clustername[idx] == 0) states[i] = 0
      else states[i] = which(clusternames == clusters$clustername[idx])
    }
    else browser()
  }
  states = .prop_states(tree, states)
  
  if(!is.null(plotfile)) pdf(plotfile)
  colors = c("darkgreen","blue","red","green","gold","purple","orange","cyan","deeppink","slategray")
  edgestates = states[tree$edge[,1]]
  edge.colors = colors[edgestates%%10+1]
  edge.colors[which(edgestates==0)] = "black"
  
  plot.phylo(tree, edge.color = edge.colors,cex = 0.25, show.tip.label = !no_labels)
  if(!is.null(plotfile)) dev.off()
}

.prop_states = function(tree, states) {
  ntips = length(tree$tip.label)
  while(length(which(states==-1))>0) {
    for(i in 1:tree$Nnode) {
      if(states[i+ntips] != -1) next
      desc = tree$edge[which(tree$edge[,1]==ntips+i),2]
      if(states[desc[1]] == -1 || states[desc[2]] == -1) next
      if(states[desc[1]] ==  states[desc[2]]) states[i+ntips] = states[desc[1]]
      else states[i+ntips] = 0
    }
  }
  states
}